@@ -7,16 +7,18 @@ use Safe;
 
 ######################################################################
 
-use vars qw( %Defaults %Filters );
-
 # Output filtering
-$Defaults{default_filters} = '';
-$Defaults{filters} = \%Filters;
-$Filters{h} = \&HTML::Entities::encode if eval { require HTML::Entities};
-$Filters{u} = \&URI::Escape::uri_escape if eval { require URI::Escape };
+use vars qw( %Filters );
+$Filters{p} = \&Text::MicroMason::Base::_printable;
+
+$Filters{h} = eval { 
+      require HTML::Entities; sub { HTML::Entities::encode( $_[0], q[<>&'"] ) } 
+    } || eval { require CGI; \&CGI::escapeHTML };
+
+$Filters{u} = eval { require URI::Escape; \&URI::Escape::uri_escape };
 
 sub defaults {
-  (shift)->NEXT('defaults'), %Text::MicroMason::Filters::Defaults
+  (shift)->NEXT('defaults'), filters => \%Filters, default_filters => ''
 }
 
 ######################################################################
@@ -27,9 +29,9 @@ sub assemble {
   my @tokens = @_;
   # warn "Filter assemble";
   foreach my $position ( 0 .. int( $#tokens / 2 ) ) {
-    if ( $tokens[$position * 2] eq 'output' ) {
+    if ( $tokens[$position * 2] eq 'expr' ) {
       my $token = $tokens[$position * 2 + 1];
-      my $filt_flags = ($token =~ s/\|\s*(\w+(?:[\s\,]+\w+)*)\s*\z//) ? $1 : '';
+      my $filt_flags = ($token =~ s/[^\|]\|\s*(\w+(?:[\s\,]+\w+)*)\s*\z//) ? $1 : '';
       my @filters = $self->parse_filters($self->{default_filters}, $filt_flags);
       if ( @filters ) {
 	$token = '$m->filter( ' . join(', ', map "'$_'", @filters ) . ', ' . 
@@ -38,7 +40,6 @@ sub assemble {
       $tokens[$position * 2 + 1] = $token;
     }
   }
-  
   $self->NEXT('assemble', @tokens );
 }
 
@@ -47,9 +48,9 @@ sub parse_filters {
   my $self = shift;
   
   my $no_ns;
+  my $short = join '', 'n', grep { length($_) == 1 } keys %{ $self->{filters} };
   reverse grep { not $no_ns ||= /^n$/ } reverse
-    map { /^[hun]{2,5}$/ ? split('') : split(/[\s\,]+/) } 
-	@_;
+    map { /^[$short]{2,5}$/ ? split('') : split(/[\s\,]+/) } @_;
 }
 
 ######################################################################
@@ -66,8 +67,7 @@ sub filter_functions {
   } elsif ( scalar @_ == 1 ) {
     my $key = shift;
     if ( ! ref $key ) {
-      $filters->{ $key } || 
-	  $self->croak_msg("No definition for a filter named '$key'" )
+      $filters->{ $key }
     } else {
       @{ $filters }{ @$key }
     }
@@ -76,18 +76,25 @@ sub filter_functions {
   }
 }
 
+# @functions = $mason->get_filter_functions( @flags_or_functions );
+sub get_filter_functions {
+  my $self = shift;
+  
+  map {
+    ( ref $_ eq 'CODE' ) ? $_ : $self->{filters}{ $_ }  
+	or $self->croak_msg("No definition for a filter named '$_'" );
+  } @_ 
+}
+
 # $result = $mason->filter( @filters, $content );
 sub filter {
   my $self = shift;
-  my $content = pop;
+  local $_ = pop;
   
-  foreach my $filter ( @_ ) {
-    my $function = ( ref $filter eq 'CODE' ) ? $filter : 
-	$self->{filters}{ $filter } || 
-	  $self->croak_msg("No definition for a filter named '$filter'" );
-    $content = &$function($content)
+  foreach my $function ( $self->get_filter_functions( @_ ) ) {
+    $_ = &$function($_)
   }
-  $content
+  $_
 }
 
 ######################################################################
@@ -110,6 +117,11 @@ Instead of using this class directly, pass its name to be mixed in:
     use Text::MicroMason;
     my $mason = Text::MicroMason->new( -Filters );
 
+Use the standard compile and execute methods to parse and evalute templates:
+
+  print $mason->compile( text=>$template )->( @%args );
+  print $mason->execute( text=>$template, @args );
+
 Enables filtering of template expressions using HTML::Mason's conventions:
 
     <%args> $name </%args>
@@ -138,7 +150,9 @@ You can define additional filters and stack them:
 
 This module enables the filtering of expressions before they are output, using HTML::Mason's "|hun" syntax.
 
-If you have HTML::Entities and URI::Escape available they are loaded to provide the default "h" and "u" filters. If those modules can not be loaded, no error message is produced but any subsequent use of them will fail with a message stating "No definition for a filter named 'h'".
+If you have HTML::Entities and URI::Escape available they are loaded to provide the default "h" and "u" filters. If those modules can not be loaded, no error message is produced but any subsequent use of them will fail.
+
+Attempted use of an unknown filter name will croak with a message stating "No definition for a filter named 'h'".
 
 =head2 Public Methods
 
@@ -161,20 +175,6 @@ If called with one or more pairs of filter flags and associated functions, adds
 
   $mason->filter_functions( $flag => $function, ... );
 
-=item parse_filters
-
-Parses one or more strings containing any number of filter flags and returns a list of flags to be used. 
-
-  @flags = $mason->parse_filters( @filter_strings );
-
-Flags should be separated by commas, except that the commas may be omitted when using only the built-in "h", "u" and "n" flags. Flags are applied from left to right. Any use of the "n" flag wipes out all flags defined to the left of it. 
-
-=item filter
-
-Applies one or more filters to the provided content string.
-
-  $result = $mason->filter( @filters, $content );
-
 =back
 
 =head2 Supported Attributes
@@ -195,6 +195,26 @@ Optional comma-separated string of filter flags to be applied to all output expr
 
 This method goes through the lexed template tokens looking for uses of filter flags, which it then rewrites as appropriate method calls before passing the tokens on to the superclass.
 
+=item parse_filters
+
+Parses one or more strings containing any number of filter flags and returns a list of flags to be used. 
+
+  @flags = $mason->parse_filters( @filter_strings );
+
+Flags should be separated by commas, except that the commas may be omitted when using a combination of single-letter flags. Flags are applied from left to right. Any use of the "n" flag wipes out all flags defined to the left of it. 
+
+=item get_filter_functions
+
+Accepts filter flags or function references and returns a list of the corresponding functions. Dies if an unknown filter flag is used.
+
+  @functions = $mason->get_filter_functions( @flags_or_functions );
+
+=item filter
+
+Applies one or more filters to the provided content string.
+
+  $result = $mason->filter( @flags_or_functions, $content );
+
 =back
 
 
@@ -202,10 +222,10 @@ This method goes through the lexed template tokens looking for uses of filter fl
 
 For an overview of this templating framework, see L<Text::MicroMason>.
 
-This is a mixin class intended for use with L<Text::MicroMason::Mason>.
+This is a mixin class intended for use with L<Text::MicroMason::HTMLMason>.
 
 For distribution, installation, support, copyright and license 
-information, see L<Text::MicroMason::ReadMe>.
+information, see L<Text::MicroMason::Docs::ReadMe>.
 
 =cut
 
